# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def get_data_lock(path, fn_name):
    """
    Get and initialize and set the lock on 'data_update.log' in path.

    If path does not already exist and the lock file is not empty then a lock
    is not set and the returned value is None.

    When a lock is set the lock file will contain the user, hostname, pid,
    date, and time.

    The opened file descriptor holding the lock is returned.

    The caller is responsible for closing the returned file descriptor to release the
    lock. The lock file should be truncated if everything went as expected.

    This function is intended for internal casaconfig use.

    Parameters
       - path (str) - The location where 'data_update.log' is to be found.
       - fn_name (str) - A string giving the name of the calling function to be recorded in the lock file.
    Returns
       - the open file descriptor holding the lock. Close this file descriptor to release the lock. Returns None if path does not exist or the lock file is not empty.

    """

    import fcntl
    import os
    import getpass
    from datetime import datetime

    if not os.path.exists(path): return None

    lock_path = os.path.join(path, 'data_update.lock')
    lock_exists = os.path.exists(lock_path)

    # open and lock the lock file - don't truncate the lock file here if it already exists, wait until it's locked
    mode = 'r+' if os.path.exists(lock_path) else 'w'
    lock_fd = open(lock_path, mode)
    fcntl.lockf(lock_fd, fcntl.LOCK_EX)

    # see if the lock file is empty
    if (lock_exists):
        lockLines = lock_fd.readlines()
        if (len(lockLines) > 0) and (len(lockLines[0]) > 0):
            # not empty
            lock_fd.close()
            return None

    # write the lock information, the seek and truncate are probably not necessary
    try:
        lock_fd.seek(0)
        lock_fd.truncate(0)
        lock_fd.write("locked using %s by %s on %s : pid = %s at %s" % (fn_name, getpass.getuser(), os.uname().nodename, os.getpid(), datetime.today().strftime('%Y-%m-%d:%H:%M:%S')))
        lock_fd.flush()
    except Exception as exc:
        print("ERROR! Unexpected failure in writing lock information to lock file %s" % lock_path)
        print("ERROR! Called by function : %s" % fn_name)
        print("ERROR! : %s" % exc)
        lock_fd.close()
        return None

    return lock_fd
   



# An example site config file.
# Place this in a location checked by casaconfig :
#   /opt/casa/casasiteconfig.py
#   /home/casa/casasiteconfig.py
#   the environment value CASASITECONFIG - use the fully qualified path
#   anywhere in the python path, e.g. the site-packages directory in the CASA being used.
#

# This file should be edited to set measurespath as appropriate

# Set this to point to the location where the site maintained casarundata can be found
# by default datapath will include measurespath

measurespath = "/path/to/installed/casarundata"

# turn off all auto updates of data

measures_auto_update = False
data_auto_update = False



import sys
import os
from casaconfig.private.get_argparser import get_argparser

parser = get_argparser(add_help=True)

# get_argparser supplies configfile, noconfig, nositeconfig
# add measurespath, pull-data, data-update, measures-update, update-all, reference-testing, current-data

parser.prog = "casaconfig"

# measurespath will default to the value in config if not set here
parser.add_argument( "--measurespath", dest='measurespath', default=None,
                     help="location of casarundata")

parser.add_argument( "--pull-data", dest='pulldata', action='store_const', const=True, default=False,
                     help="invoke pull_data() to populate measurespath with latest casarundata")
parser.add_argument( "--data-update", dest='dataupdate', action='store_const', const=True, default=False,
                     help="invoke data_update() to update measurespath to the latest casarundata")
parser.add_argument( "--measures-update", dest='measuresupdate', action='store_const', const=True, default=False,
                     help="invoke measures_update() to update measurespath to the latest measures data")
parser.add_argument( "--update-all", dest='updateall', action='store_const', const=True, default=False,
                     help="invoke update_all() to populate (update) measurespath with the latest casarundata and measures data")
parser.add_argument( "--reference-testing", action='store_const', const=True, dest='referencetesting', default=False,
                     help="set measurespath to the casarundata when this version was produced, used for testing purposes")
parser.add_argument( "--current-data", dest='currentdata', action='store_const', const=True, default=False,
                     help="print out a summary of the current casarundata and measures data installed in measurespath and then exit")
parser.add_argument("--force", dest='force', action='store_const', const=True, default=False,
                    help="force an update using the force=True option to update_all, data_update, and measures_update")

# initialize the configuration to be used
flags,args = parser.parse_known_args(sys.argv)
from casaconfig import config

# import the casaconfig module
import casaconfig

# make sure measurespath reflects any command line value
if flags.measurespath is None:
    flags.measurespath = config.measurespath

# watch for measurespath of None, that likely means that casasiteconfig.py is in use and this has not been set. It can't be used then.
if flags.measurespath is None:
    print("measurespath has been set to None, likely in casasiteconfig.py.")
    print("Either provide a measurespath on the casaconfig command line or edit casasiteconfig.py or other a user config.py to set measurespath to a location.")
    sys.exit(1)

# do any expanduser and abspath - this is what should be used
measurespath = os.path.abspath(os.path.expanduser(flags.measurespath))

if flags.currentdata:
    if not os.path.exists(measurespath) or not os.path.isdir(measurespath):
        print("No data installed at %s. The measurespath does not exist or is not a directory." % measurespath)
    else:
        from casaconfig import get_data_info
        print("current data installed at %s" % measurespath)
        dataInfo = get_data_info(measurespath)
        
        # casarundata
        casarunInfo = dataInfo['casarundata']
        if casarunInfo is None or casarunInfo['version'] == "invalid":
            print("No casarundata found (missing or unexpected readme.txt contents, not obviously legacy casa data).")
        elif casarunInfo['version'] == "unknown":
            print("casarundata version is unknown (probably legacy casa data not maintained by casaconfig).")
        else:
            currentVersion = casarunInfo['version']
            currentDate = casarunInfo['date']
            print('casarundata version %s installed on %s' % (currentVersion, currentDate))
            
        # measures
        measuresInfo = dataInfo['measures']
        if measuresInfo is None or measuresInfo['version'] == "invalid":
            print("No measures data found (missing or unexpected readme.txt, not obviously legacy measures data).")
        elif measuresInfo['version'] == "unknown":
            print("measures version is unknown (probably legacy measures data not maintained by casaconfig).")
        else:
            currentVersion = measuresInfo['version']
            currentDate = measuresInfo['date']
            print('measures version %s installed on %s' % (currentVersion, currentDate))
 
    # ignore any other arguments

else:
    if flags.referencetesting:
        print("reference testing using pull_data and 'release' version into %s" % measurespath)
        casaconfig.pull_data(measurespath,'release')
        # ignore all other options
    else:
        # the update options, update all does everything, no need to invoke anything else
        print("Checking for updates into %s" % measurespath)
        if flags.updateall:
            casaconfig.update_all(measurespath,force=flags.force)
        else:
            # do any pull_update first
            if flags.pulldata:
                casaconfig.pull_data(measurespath)
            # then data_update, not necessary if pull_data just happened
            if flags.dataupdate and not flags.pulldata:
                casaconfig.data_update(measurespath, force=flags.force)
            # then measures_update
            if flags.measuresupdate:
                casaconfig.measures_update(measurespath, force=flags.force)

sys.exit(0)



# lsit of paths where CASA should search for data subdirectories. Default [measurespath].
datapath = [ ]

# location of required measures data, takes precedence over any measures data also present in datapath.
measurespath = "~/.casa/data"

# automatically update measures data if not current (measurespath must be owned by the user)
# when data_auto_update is True then measures_auto_update MUST also be True
measures_auto_update = True

# automatically update casarundata and measures data if not current (measurespath must be owned by the user)
data_auto_update = True

# location of the optional user's startup.py
startupfile = '~/.casa/startup.py'

# location of the cachedir
cachedir = '~/.casa'

# log file path/name
logfile='casa-%s.log' % _time.strftime("%Y%m%d-%H%M%S", _time.gmtime())

# do not create a log file when True, If True, then any logfile value is ignored and there is no log file
nologfile = False

# print log output to terminal when True (in addition to any logfile and CASA logger)
log2term = False

# do not start the CASA logger GUI when True
nologger = False

# avoid starting GUI tools when True. If True then the CASA logger is not started even if nologger is False
nogui = False

# the IPython prompt color scheme. Must be one of "Neutral", "NoColor", "Linux" or "LightBG", default "Neutral"
# if an invalid color is given a warning message is printed and logged but CASA continues using the default color
colors = "Neutral"

# startup without a graphical backend if True
agg = False

# attempt to load the pipeline modules and set other options appropriate for pipeline use if True
# when pipeline is True then agg will be assumed to be true even if agg is set to False here or on the command line
pipeline = False

# create and use an IPython log using the iplogfile path
iplog = False

# the IPython log file path name to be used when iplog is True
iplogfile='ipython-%s.log' % _time.strftime("%Y%m%d-%H%M%S", _time.gmtime())

# include the user's local site-packages in the python path if True. May conflict with CASA modules
user_site = False



# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def read_readme(path):
    """
    Read and parse the contents of a readme.txt file used by casaconfig.

    A dictionary containing the 'version', 'date', and 'extra' (containing
    a list of optional extra lines found). On error, the return values is None.

    The extra lines are stripped and do not include lines begining with '#'

    The format is assumed to be:
        a line begining with #, which is ignored.
        a line "version : the versions string"
        a line "date : the date"
        optional extra lines (the manifest of installed files for the main readme)

    The version string and date are stripped of leading and trailing whitespace.

    Parameters
       - path (str) - the path to the file to be read

    Returns
       Dictionary of 'version' (the version string), 'date' (the date string),
       'extra' (a list of any extra lines found). The return value is None on error.
    """

    import os

    version = ""
    date = ""
    extra = []
    result = None
    
    try:
        with open(path, 'r') as fid:
            readmeLines = fid.readlines()
            version = readmeLines[1].split(':')[1].strip()
            date = readmeLines[2].split(':')[1].strip()
            if len(readmeLines) > 3:
                for extraLine in readmeLines[3:]:
                    if (extraLine[0] != '#'):
                        extra.append(extraLine.strip())
            result = {'version':version, 'date':date, 'extra':extra}
    except:
        pass

    return result



########################################################################
#
# Copyright (C) 2022
# Associated Universities, Inc. Washington DC, USA.
#
# This script is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
# License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#
# Correspondence concerning AIPS++ should be adressed as follows:
#        Internet email: aips2-request@nrao.edu.
#        Postal address: AIPS++ Project Office
#                        National Radio Astronomy Observatory
#                        520 Edgemont Road
#                        Charlottesville, VA 22903-2475 USA
#
########################################################################
'''
Default values for the configuration variables for all CASA python packages.
DO NOT ADD new configuration variables here. Instead, add them in
config_defaults_static.py (found in the same directory as this file).
'''
import os as _os
import sys as _sys
import time as _time
import pkgutil as _pkgutil

from .get_argparser import get_argparser as __get_argparser

## get the ArgumentParser with the arguments needed by casaconfig, help is turned off
## this is used to supply command line configuration variales to the static defaults
## specification.
__parser = __get_argparser()
__flags,__args = __parser.parse_known_args(_sys.argv)

def _globals( ):
    return globals()

exec( open(_os.path.join(_os.path.dirname(__file__),'config_defaults_static.py')).read( ), globals( ) )

